from matplotlib import rc
from matplotlib.ticker import EngFormatter
import numpy as np
from os import path
import subprocess
import sys
import winreg
import sip

# Qt4 bindings for core Qt functionalities (non-GUI)
from PyQt4 import QtCore
# Python Qt4 bindings for GUI objects
from PyQt4 import QtGui

# import the MainWindow widget from the converted .ui file
from ui_CompareWaveforms import Ui_MainWindow

# Registry access ===============================================================
REG_PATH = r"SOFTWARE\CompareWaveforms\Settings"

def set_reg(key, value):
    try:
        winreg.CreateKey(winreg.HKEY_CURRENT_USER, REG_PATH)
        registry_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, REG_PATH, 0,
                                       winreg.KEY_WRITE)
        winreg.SetValueEx(registry_key, key, 0, winreg.REG_SZ, value)
        winreg.CloseKey(registry_key)
        return True
    except WindowsError:
        return False

def get_reg(name):
    try:
        registry_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, REG_PATH, 0,
                                       winreg.KEY_READ)
        value, regtype = winreg.QueryValueEx(registry_key, name)
        winreg.CloseKey(registry_key)
        return value
    except WindowsError:
        return ""
# ==============================================================================

class CompareMainWindow(QtGui.QMainWindow, Ui_MainWindow):
    # Data storage
    lts_time = np.array([])
    lts_time_display = np.array([])
    lts_ampl = np.array([])
    data_time = np.array([])
    data_ampl = np.array([])
    # Slider increment
    slider_incr = float

    def __init__(self, parent=None):
        # initialization of the superclass
        super(CompareMainWindow, self).__init__(parent)
        # setup the GUI --> function generated by pyuic
        self.setupUi(self)

        # Restore last file & path names or empty strings
        string = get_reg("LastLtsFilename")
        self.lineEditLtsFile.setText(string)
        string = get_reg("LastScopeFilename")
        self.lineEditScopeFile.setText(string)
        string = get_reg("LastPlotPath")
        self.lineEditPlotPath.setText(string)

        # connect the signals with the slots
        QtCore.QObject.connect(self.pushButtonSave, QtCore.SIGNAL("clicked()"),
                               save_plot)
        QtCore.QObject.connect(self.pushButtonCompare, QtCore.SIGNAL("clicked()"),
                               read_input)
        QtCore.QObject.connect(self.sliderSkew, QtCore.SIGNAL('valueChanged(int)'),
                               slider)
        QtCore.QObject.connect(self.pushButtonLtsFile, QtCore.SIGNAL('clicked()'),
                               file_ltspice)
        QtCore.QObject.connect(self.pushButtonScopeFile, QtCore.SIGNAL('clicked()'),
                               file_scope)
        QtCore.QObject.connect(self.pushButtonPlotPath, QtCore.SIGNAL('clicked()'),
                               path_plot)

        # Get the figure instance to which the axes belong
        self.fig = self.mpl.canvas.ax.get_figure()
        # Save shorthand to this axes instance
        self.ax = self.mpl.canvas.ax

        # This seems to be necessary to prevent python crashes on exit
        sip.setdestroyonexit(False)


def read_input():
    # Read and store data files
    # Get time & ampl from LTspice waveform export file (one header line,
    # whitespace-separated values
    mw.lts_file = mw.lineEditLtsFile.text()
    mw.lts_time, mw.lts_ampl = np.loadtxt(mw.lts_file, skiprows=1).T
    mw.lts_time_display = mw.lts_time

    # Get time & ampl from DSO7104 scope type file (2 header lines, CSV), may have
    # time offset.
    # Capture only one channel.
    mw.data_file = mw.lineEditScopeFile.text()
    mw.data_time, mw.data_ampl = np.loadtxt(mw.data_file, skiprows=2, delimiter=",").T
    # Remove offset from scope data
    scope_offset = mw.data_time[0]
    mw.data_time -= scope_offset

    # Calculate slider offset value
    # LTspice time array runs from zero to max, so max = last element. Scale so that
    # slider = +/-10% of range
    mw.slider_incr = mw.lts_time[-1] * 0.1 / mw.sliderSkew.maximum()
    update_plot()


def file_ltspice():
    start = path.dirname(mw.lineEditLtsFile.text())
    file = QtGui.QFileDialog.getOpenFileName(mw, caption="LTspice File", directory=start,
                                             filter="Text files (*.txt)")
    if file:
        file = path.normpath(file)
        mw.lineEditLtsFile.setText(file)
        set_reg("LastLtsFilename", file)


def file_scope():
    start = path.dirname(mw.lineEditScopeFile.text())
    file = QtGui.QFileDialog.getOpenFileName(mw, caption="Scope File", directory=start,
                                             filter="CSV files (*.csv)")
    if file:
        file = path.normpath(file)
        mw.lineEditScopeFile.setText(file)
        set_reg("LastScopeFilename", file)


def path_plot():
    start = mw.lineEditPlotPath.text()
    path = QtGui.QFileDialog.getExistingDirectory(mw, directory=start)
    if path:
        # path.normpath(path) doesn't work right with getExistingDirectory() output.
        # reason? fcn returns '\' instead of '/'
        mw.lineEditPlotPath.setText(path)
        set_reg("LastPlotPath", path)


def save_plot():
    # Output file names
    png_file = mw.lineEditPlotPath.text() + r"\ComparePlot.png"
    # Save the plot with boundaries "tight" to the axes (no wasted space)
    mw.fig.savefig(png_file, bbox_inches="tight")


def slider(value):
    if mw.lts_time.size == 0 or mw.data_time.size == 0:
        # Slider can't operate on empty data
        return
    # Adjust display time axis based on slider position
    mw.lts_time_display = mw.lts_time + value * mw.slider_incr
    update_plot()


def update_plot():
    mw.ax.clear()
    # X-axis time parameters
    mw.ax.set_xlabel("TIME, s")
    mw.ax.xaxis.grid(color='gray', linestyle='dashed')
    # Y-axis amplitude parameters
    mw.ax.set_ylabel("AMPLITUDE, V", color="k")
    mw.ax.yaxis.grid(color='gray', alpha=0.5, linestyle='dashed')
    for label in mw.ax.get_yticklabels():
        label.set_color("k")

    # Engineering units for freq axis
    formatter = EngFormatter(places=1)
    mw.ax.xaxis.set_major_formatter(formatter)
    # Plot LTspice output and data
    sim_line, = mw.ax.plot(mw.lts_time_display, mw.lts_ampl, "skyblue")
    data_line, = mw.ax.plot(mw.data_time, mw.data_ampl, "blue")
    # Put up a legend
    mw.ax.legend([sim_line, data_line], ["Sim", "Data"], bbox_to_anchor=[0.5, 1.05],
                 loc='center', ncol=2)
    mw.ax.autoscale_view(tight=True)
    mw.mpl.canvas.draw()


# Set an axis font which includes Greek letters
rc('font', **{'family': 'sans-serif', 'sans-serif': ['Tahoma'], 'size': '12'})

# create the GUI application
app = QtGui.QApplication(sys.argv)
# instantiate the main window
mw = CompareMainWindow()
mw.show()
# Start the Qt main loop execution, exiting from this script
# with the same return code of Qt application
sys.exit(app.exec_())
